class MERClassicReport(PWReportPage):
	def init(self):
		super(MERClassicReport, self).init()
		self.project		= None
		self.withObjective 	= True
		self.withMilestones = True
		self.withUpcoming 	= True
		self.withOverdues 	= True
		self.withDeadlines 	= True
		self.withProjectColumn = False
		self.withInformation= True
		self.withIssues		= True
		self.withFiles		= True
		self.withRisks 		= True
		self.withResources	= True
		self.withTimes 		= True
		self.duration		= None
		self.iterItem		= None
		self.iterIndex		= 0
		self.milestonesSortDescriptors			= [NSSortDescriptor.alloc().initWithKey_ascending_("expectedEndDate", True)]
		self.upcomingActivitiesSortDescriptors	= [NSSortDescriptor.alloc().initWithKey_ascending_("expectedStartDate", True)]
		self.overdueActivitiesSortDescriptors	= [NSSortDescriptor.alloc().initWithKey_ascending_("expectedStartDate", True)]
		self.deadlineActivitiesSortDescriptors	= [NSSortDescriptor.alloc().initWithKey_ascending_("expectedStartDate", True)]
		self.infoElementsSortDescriptors		= [NSSortDescriptor.alloc().initWithKey_ascending_("informationDate", True)]
		self.issuesSortDescriptors				= [NSSortDescriptor.alloc().initWithKey_ascending_("fixUntilDate", True)]
		self.risksSortDescriptors				= [NSSortDescriptor.alloc().initWithKey_ascending_("recordingDate", True)]
		self.resourcesSortDescriptors			= [NSSortDescriptor.alloc().initWithKey_ascending_("title", True)]
		self.filesSortDescriptors				= [NSSortDescriptor.alloc().initWithKey_ascending_("title", True)]
		return self

	def internalParameterKeys(self):
		return ["milestonesSortDescriptors", "upcomingActivitiesSortDescriptors", "overdueActivitiesSortDescriptors", "deadlineActivitiesSortDescriptors", "issuesSortDescriptors", "infoElementsSortDescriptors", "risksSortDescriptors", "resourcesSortDescriptors", "filesSortDescriptors"]

	def milestones(self):
		activities = self.project.linkedSubActivitiesDeepIncludeMe()
		predicate = NSPredicate.predicateWithFormat_("isMilestone = TRUE AND isPrivate = FALSE AND isAssignment = NO AND mayBeSeenByCurrentUser = YES")
		milestones = activities.filteredArrayUsingPredicate_(predicate)
		return milestones.sortedArrayUsingDescriptors_(self.milestonesSortDescriptors)
		
	def upcomingActivities(self):	
		result = []
		minStart = NSDate.systemNowInGMT()
		maxStart = self.duration.dateFloatedFromDate_(minStart)
		
		for activity in self.project.visibleSimpleActivities():
			start = activity.expectedStartDate()
			if (not activity.isPrivate()) and (not activity.isMilestone()) and start>=minStart and start<=maxStart and (not activity.isComplete()):
				result.append(activity)
		return NSArray.arrayWithArray_(result).sortedArrayUsingDescriptors_(self.upcomingActivitiesSortDescriptors)
		
	def overdueActivities(self):
		result = []
		now = NSDate.systemNowInGMT()
		for activity in self.project.visibleSimpleActivities():
			start = activity.expectedStartDate()
			end = activity.expectedEndDate()
			if (not activity.isPrivate()) and ((now>=end and (not activity.isComplete())) or (now>=start and activity.actualCompletion()==0)):
				result.append(activity)
		return NSArray.arrayWithArray_(result).sortedArrayUsingDescriptors_(self.overdueActivitiesSortDescriptors)

	def deadlineActivities(self):
		result = []
		minEnd = NSDate.systemNowInGMT()
		maxEnd = self.duration.dateFloatedFromDate_(minEnd)
		for activity in self.project.visibleSimpleActivities():
			end = activity.expectedEndDate()
			if (not activity.isPrivate()) and end>=minEnd and end<=maxEnd and (not activity.isComplete()):
				result.append(activity)
		return NSArray.arrayWithArray_(result).sortedArrayUsingDescriptors_(self.deadlineActivitiesSortDescriptors)
	
	def issues(self):
		predicate = NSPredicate.predicateWithFormat_argumentArray_("entity.name = %@ AND isPrivate = FALSE AND flagStatus <> 1", ["Issue"])
		issues = self.project.deepElements().allObjects().filteredArrayUsingPredicate_(predicate)
		return issues.sortedArrayUsingDescriptors_(self.issuesSortDescriptors)
	
	def infoElements(self):
		predicate = NSPredicate.predicateWithFormat_argumentArray_("entity.name = %@ AND isPrivate = FALSE", ["Information"])
		infos = self.project.deepElements().allObjects().filteredArrayUsingPredicate_(predicate)
		return infos.sortedArrayUsingDescriptors_(self.infoElementsSortDescriptors)	
		
	def risks(self):
		predicate = NSPredicate.predicateWithFormat_argumentArray_("entity.name = %@ AND isPrivate = FALSE AND (parentActivity.isComplete = FALSE OR parentActivity = nil)", ["Risk"])
		risks = self.project.deepElements().allObjects().filteredArrayUsingPredicate_(predicate)
		return risks.sortedArrayUsingDescriptors_(self.risksSortDescriptors)	

	def resources(self):
		predicate = NSPredicate.predicateWithFormat_argumentArray_("isPrivate = FALSE", None)
		resources = self.project.visibleResources().allObjects().filteredArrayUsingPredicate_(predicate)
		resources = resources.sortedArrayUsingDescriptors_(self.resourcesSortDescriptors)
		return resources	

	def files(self):
		predicate = NSPredicate.predicateWithFormat_argumentArray_("entity.name = %@ AND isPrivate = FALSE AND versions.@count>0", ["File"])
		files = self.project.deepElements().allObjects().filteredArrayUsingPredicate_(predicate)
		files = files.sortedArrayUsingDescriptors_(self.filesSortDescriptors)
		return files
	
	def showObjective(self):
		return self.withObjective and self.project.itemDescription()

	def showMilestones(self):
		return self.withMilestones and len(self.milestones())>0

	def showUpcoming(self):
		return self.withUpcoming and len(self.upcomingActivities())>0

	def showOverdues(self):
		return self.withOverdues and len(self.overdueActivities())>0
	
	def showDeadlines(self):
		return self.withDeadlines and len(self.deadlineActivities())>0

	def showInformation(self):
		return self.withInformation and len(self.infoElements())>0

	def showIssues(self):
		return self.withIssues and len(self.issues())>0

	def showRisks(self):
		return self.withRisks and len(self.risks())>0
	
	def showResources(self):
		return self.withResources and len(self.resources())>0

	def showFiles(self):
		return self.withFiles and len(self.files())>0

	def rowClass(self):
		if self.iterIndex%2:
			return "even"

	def listItemClass(self):
		if self.iterIndex == 0:
			return "first"
	
	def dateFormat(self):
		if self.withTimes:
			return "medium/short"
		else:
			return "medium"

	def mailTo(self):
		if self.iterItem and self.iterItem.email():
			return "mailto:"+self.iterItem.email()

	def navigationItems(self):
		bundle = NSBundle.bundleWithIdentifier_("net.projectwizards.reports.Classic")
		items = []
		if self.showMilestones():
			items.append({"href":  "#milestones", "title": NSLocalizedStringFromTableInBundle("Milestones", None, bundle, "")})
		if self.showUpcoming():
			items.append({"href":  "#upcoming", "title": NSLocalizedStringFromTableInBundle("Upcoming", None, bundle, "")})
		if self.showOverdues():
			items.append({"href":  "#overdues", "title": NSLocalizedStringFromTableInBundle("Overdues", None, bundle, "")})
		if self.showDeadlines():
			items.append({"href":  "#deadlines", "title": NSLocalizedStringFromTableInBundle("Deadlines", None, bundle, "")})
		if self.showInformation():
			items.append({"href":  "#information", "title": NSLocalizedStringFromTableInBundle("Information", None, bundle, "")})
		if self.showIssues():
			items.append({"href":  "#issues", "title": NSLocalizedStringFromTableInBundle("Issues", None, bundle, "")})
		if self.showRisks():
			items.append({"href":  "#risks", "title": NSLocalizedStringFromTableInBundle("Risks", None, bundle, "")})
		if self.showResources():
			items.append({"href":  "#resources", "title": NSLocalizedStringFromTableInBundle("Resources", None, bundle, "")})
		if self.showFiles():
			items.append({"href":  "#files", "title": NSLocalizedStringFromTableInBundle("Files", None, bundle, "")})
		return items
