class MERCostDistribution(PWReportPage):
	def init(self):
		super(MERCostDistribution, self).init()
		self.project					= None
		self.intervalUnit				= 0
		self.isCumulated				= False
		self.budgetColor				= None
		self.expectedResourceCostColor	= None
		self.elementsCostColor			= None
		self.budgetCostBalanceColor		= None
		self.iterIndex					= 0
		self.iterIntervalStartDate		= None
		return self
	
	def showObjective(self):
		return self.project.itemDescription()

	def rowClass(self):
		if self.iterIndex%2:
			return "even"

	def dateFormat(self):
		return "medium/short"
	
	def intervalUnitIsTooSmall(self):
		if self.project.expectedEndDate() and self.project.expectedStartDate():
			projectSeconds = self.project.expectedEndDate().timeIntervalSinceDate_(self.project.expectedStartDate())
			intervalSeconds = MEDuration.durationWithAmount_unit_isFloating_relativeError_(1, self.intervalUnit, 1, 0).roughSeconds()
			numberOfIntervals = projectSeconds / intervalSeconds
			return numberOfIntervals > 300
		else:
			return False
	
	def shouldIncludeStartDate(self, startDate):
		endDate =  MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, startDate, self.project.sharedNSCalendar())
		if self.project.approvedBudgetUndistributedBetweenDate_andDate_(startDate, endDate) > 0.0:
			return True
		if self.project.expectedCostBetweenDate_andDate_(startDate, endDate) > 0.0:
			return True
		return False
	
	def intervalStartDates(self):
		start = MEDuration.startOfUnit_containingDate_calendar_(self.intervalUnit, self.project.expectedStartDate(), self.project.sharedNSCalendar())
		end   = self.project.expectedEndDate()
		
		intervalStart = start
		dates = []
		
		while intervalStart<end:
			if self.shouldIncludeStartDate(intervalStart):
				dates.append(intervalStart)
			intervalStart = MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, intervalStart, self.project.sharedNSCalendar())
			
		return dates
		
	def setIterIntervalStartDate_(self, date):
		self.iterIntervalStartDate = date
		self.iterIntervalEndDate = MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, date, self.project.sharedNSCalendar())
		
		
	def iterIntervalDescription(self):
		return MEDuration.preciseDescriptionForUnit_atDate_calendar_(self.intervalUnit, self.iterIntervalStartDate, self.project.sharedNSCalendar())

	def iterIntervalCalculcationStartDate(self):
		if self.isCumulated:
			return self.project.expectedStartDate()
		else:
			return self.iterIntervalStartDate

	def iterIntervalBudget(self):
		return self.project.approvedBudgetUndistributedBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)

	def iterIntervalExpectedWorkCost(self):
		return self.project.expectedWorkCostBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)
	
	def iterIntervalElementsCost(self):
		return self.project.elementsCostBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)

	def iterIntervalBaseCost(self):
		return self.project.baseCostBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)
	
	def iterIntervalTotalCost(self):
		return self.project.expectedCostBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)

	def iterIntervalBudgetCostBalance(self):
		return self.project.budgetExpectedCostBalanceBetweenDate_andDate_(self.iterIntervalCalculcationStartDate(), self.iterIntervalEndDate)

	def budgetSum(self):
		if self.project.approvedBudgetAbsolute():
			return self.project.approvedBudgetAbsolute()
		else:
			return self.project.approvedBudgetDistributed()

	def xTicks(self):
		xTicks = "\"xTicks\": [ "
		intervalStartDates = self.intervalStartDates()
		modulo = len(intervalStartDates)/15 + 1
		for i in range(len(intervalStartDates)):
			date = intervalStartDates[i]
			if i<len(intervalStartDates)-1:
				separator = ","
			else:
				separator = ""
			if i % modulo == 0:
				description = MEDuration.preciseDescriptionForUnit_atDate_calendar_(self.intervalUnit, date, self.project.sharedNSCalendar())
			else:
				description = ""
			xTicks += "{v:%d, label:\"%s\"}%s\n" % (i, description, separator)
			
		xTicks = xTicks + "]"
		return xTicks
	
	def dataSetStringForKey(self, key):
		dataSetString = "\"%s\",[" % (key)
		intervalStartDates = self.intervalStartDates()
		for i in range(len(intervalStartDates)):
			intervalStartDate = intervalStartDates[i]
			intervalEndDate   = MEDuration.endOfUnit_withStartDate_calendar_(self.intervalUnit, intervalStartDate, self.project.sharedNSCalendar())
			if self.isCumulated:
				startDate = self.project.expectedStartDate()
			else:
				startDate = intervalStartDate

			if i<len(intervalStartDates)-1:
				separator = ","
			else:
				separator = ""
			value = self.project.distributedValueForKey_betweenDate_andDate_(key, startDate, intervalEndDate)
			if not value:
				value = 0
			dataSetString += "[%d, %s]%s" % (i, value, separator)
		dataSetString += "]"
		return dataSetString

	def budgetDataSetString(self):
		return self.dataSetStringForKey("approvedBudgetUndistributed")
	
	def expectedWorkCostDataSetString(self):
		return self.dataSetStringForKey("expectedWorkCost")

	def elementsCostDataSetString(self):
		return self.dataSetStringForKey("elementsCost")

	def budgetExpectedCostBalanceDataSetString(self):
		return self.dataSetStringForKey("budgetExpectedCostBalance")

	def budgetColorString(self):
		return self.budgetColor.webColor()

	def expectedResourceCostColorString(self):
		return self.expectedResourceCostColor.webColor()

	def elementsCostColorString(self):
		return self.elementsCostColor.webColor()

	def budgetCostBalanceColorString(self):
		return self.budgetCostBalanceColor.webColor()
